<?php
/**
 * Admin View: Statistics
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

?>


<div class="wrap about-wrap">

<h1><?php printf( esc_html__( 'Welcome to Ajax Live Search Lite', 'als' ), '' ); ?></h1>
	<div class="about-text"><?php printf( __( 'Thank you for chosing aLs. If you encounter trouble using it, you can always check out our <a href="http://ajaxlivesearch.xyz">documentation</a>. <br /> Sorry if it took too long to create the index. From now on everything will go on smoothly.', 'als' ) ); ?></div>
	<h2>You just gave your blog Google-like superpowers!</h2>
	<div class="about-text"> Below are a couple of things we would like you to know.</div>
	<h3> Intro </h3>
	<p> aLs was born out of necessity. We disected and then reconstructed around 35 searching software, isolated all the best parts and buried the rest. </p>
	<p> We then went through hundreds of search related articles, ranging from machine learning to markov chains to the basic stuffs like tf-idfs and document quality calculation.</p>
	<p> If you are interested in some of the books that we read, then send us an email to picocodes@gmail.com and I will email you back with the details. They are free books by the way, gathered from around the web. </p>
	
	<h3> How it works </h3>
	<p> To simplify things, we will use the analogy of a text book. </p>
	<p> If you open the backpage, there is a listing of important words and the pages in which they appear. </p>
	<h4> The WordPress Way (10% relevancy | 100% Slow) </h4>
	<p> If you ask WordPress to fetch pages containing the phrase "I like eating raw tomatoes"; It will go from page one to the last page, reading each individual word and checking if it contains either of the words. i.e, It will fetch every article containing either the word "i", "like", "eating" or tomatoes. </p>
	<p> It will then output the results in the order in which they were found. This is not only slow but also returns lot's of irrelevant results. Almost all of the pages will match because they contain the phrase "I", but may lack the more meaningful phrase Tomatoes.</p>

	<h3> The better Way (60% relevancy | 60% Slow)</h3>
	<p> Most (read all) plugins will take the phrase then strip out "I" and "like". This leaves them with "eating raw tomatoes". They then fetch out the results using the WordPress way and calculate the relevancy using a combination of term frequencies and domain frequencies. </p>
	<p> Don't get me wrong, the results achieve a great deal of relevancy, but it is very slow. This is because the LIKE query never uses an index. It matches word by word tyring to figure out which is relevant. </p>
	<p> Despite what they say, no index is used. They may have an index yeah, but that is useless when using a LIKE query. </p>
	
	<h3> The aLs Way (90% relevancy | 10% Slow)</h3>
	<p> Suppose the book was a health guide book; then the above method would still rank lot's of irrelevant pages just because they use the word eating and raw a lot. aLs gotcha covered!</p>
	<p> First of all, we strip out the stopwords ('I' and 'Like'). We then reverse stem our query so that we can also match the words eat and tomato.</p>
	<p> We  compare this to our previous searches log table then realise that people who search for phrases containing the word raw tend to exclude the word "-cooked". Remember our exclusion search modifier? We also notice that they tend to include the word "onion" in the same query. </p>
	<p> With this data; we reconstruct our query to look like this (no particular order) "-cook* raw eat* tomato* onion*". The * is a placeholder and means that we should fetch words that begin that way. It has no effect on our index, since we also stemmed the words found in the index at the time of creation. Hence a smaller and brighter index.</p>
	<p> We now open the backpage and pick the word cook, then take a look at the pages that it refers to. We then search for the other four words and take not of some factors such as how many pages contain them and how many times they appear in those pages. </p>
	<p> We notice that the word "eat" appears in more than 50% of the articles so we assign it a weight of 0. We also notice that the word tomato appear in 1% while onion and raw appear in 5% each. But since onion is a secondary keyword, we give it a weight of 0.5, creating the illusion that it appears in 10%. </p>
	<p> We now use this data to calculate the weight of each word in each page that contains it; unless that page also contains the word cook in which case we give it an overall weight of 0. </p>
	<p> This is done by take the log of (100/%ge of documents containing it) then multiplying that with the no. times it appears in the page. </p>
	<p> We then multiply our results with linear regression values of factors like how old it is, it's comment count and it's quality score. </p>
	<p> We merge the final results and sort the results by the calculated relevancey score to achieve a highly accurate relevancy score.</p>
	<p> I know you probably think that this is time consuming, but only 10% of the above process is done during search time. </P>
	<p> Remember that time that you installed  the plugin and thought that your website was hanging? We were doing the magic. Everything was being done in native C++, using inbuilt MySQL indexing capabilities. The relevancy of each word in respect to each page was being calculated. And instead of creating our own indexing engine, we decided to use something that is prebuilt. </p>
	<p> When we tested several plugins, on a database containing 1 million wikipedia articles and running on an old pc with 512 mb ram; most of the queries took less than 1 second to fetch results. Compared to some plugins that were taking up to 2 days to index the data and 30 mins to fetch results for a 4 word query; I think those books really worked. </p>
	<p> This plus the fact that aLs pro caches results means that you can comfortably run it on data containing several million records. </p>
	
	<h3> The Google Way (100% relevancy | 0% Slow)</h3>
	<p> Google's process is pretty simple. The problem is... no one knows how to recreate it. </p>
	<h2> Speaking of Pro!</h2>
	<div class="about-text"> You can improve your site further by <a href="https://pay.paddle.com/checkout/506334" class="button button-secondary">Going Pro</a> and getting the following benefits.</div>
	<ol>
		<li> Ability to show sponsored results(affiliate products) at the top of your search results.</li>
		<li> Save statistics in csv format for offline manipulation in your favourite spreadsheet program.</li>
		<li> View statistics in form of graphs.</li>
		<li> Double your results speed by caching query results. </li>
		<li> Fetch query suggestions from Google/ YouTube instead of your database hence less load. </li>
		<li> Better search algorithym. </li>
	</ol>
	
	<a href="https://pay.paddle.com/checkout/506334" class="button button-primary">Go Pro!</a>
</div>
